unit fingerThrd;

interface

uses
  Windows, Messages, Classes, WinSock, WSocket, Types, Datast, SysUtils,
  Dialogs;

type
  TFingerThread = class(TThread)
  private
    FClientSocket  : TWSocket; // Client using finger
    FClientHSocket : TSocket;
    FRequest       : String;
 // Client procedures
    procedure ServerWSocketDataAvailable(Sender: TObject; Error: Word);
    procedure ServerWSocketSessionClosed(Sender: TObject; Error: Word);
// Our procedures
    procedure SendAnswer;
  protected
    procedure Execute; override;
  public
    procedure   Release;
    procedure   SetSocket(ClientHSocket : TSocket);
    constructor Create;
    destructor  Destroy; override;
  end;
implementation

const 
 WM_QUIT           = $0012; // It was traken from messages.pas
 WM_USER           = $0400;

procedure ProcessMessages;
var Msg: TMsg;
begin
 while PeekMessage(Msg, 0, 0, 0, PM_REMOVE) do
 if Msg.Message <> WM_QUIT then
 begin
   TranslateMessage(Msg);
   DispatchMessage(Msg);
 end else break;
end;

procedure TFingerThread.ServerWSocketSessionClosed(
    Sender: TObject; Error: Word);
begin
  PostMessage(FClientSocket.Handle, WM_QUIT, 0, 0); // Close Client socket
  Self.Terminate; // terminate thread
end;

procedure TFingerThread.ServerWSocketDataAvailable(Sender: TObject; Error: Word);
var
   Count : Integer;
   Buffer: ShortString;
begin
   Count := (Sender as TWSocket).Receive(@Buffer[1], High(Buffer));
   if Count > 0 then
   begin
      Buffer[0] := chr(Count);
      FRequest := FRequest + Buffer;
      Count := Length(FRequest);
      if (Count >= 2) and ((FRequest[Count-1] = #$D) and (FRequest[Count] = #$A)) then
      SendAnswer;
      ProcessMessages;
   end;
end;


constructor TFingerThread.Create;
begin
  FreeOnTerminate := TRUE;
  inherited Create(TRUE);
end;

destructor TFingerThread.Destroy;
begin
  inherited Destroy;
end;

procedure TFingerThread.Release;
begin
  PostMessage(FClientSocket.Handle, WM_QUIT, 0, 0);
  ProcessMessages;
  Terminate;
end;

procedure TFingerThread.SetSocket(ClientHSocket : TSocket);
begin
  FClientHSocket  := ClientHSocket; //client socket
end;

procedure TFingerThread.Execute; // Here we start thread executing
var Msg: TMsg;
begin
  while not Terminated do // while thread not terminated
  begin
    FRequest                      := ''; // No request yet

    FClientSocket                 := TWSocket.Create(nil);

    FClientSocket.Dup(FClientHSocket);
    FClientSocket.OnDataAvailable := ServerWSocketDataAvailable;
    FClientSocket.OnSessionClosed := ServerWSocketSessionClosed;

    while GetMessage(msg, 0, 0, 0) and (not Terminated) do // Message loop
    begin
      TranslateMessage(msg);
      DispatchMessage(msg);
    end;

    FClientSocket.Close; // Close all we opened before
    FClientSocket.Free;

    ThreadCache.LockList;
    ThreadCache.Add(self);
    ThreadCache.UnlockList;
    Suspend;
  end;
end;
procedure TFingerThread.SendAnswer;
var Result: String;
    UInfo: TUserInfo;
    I, O, W, S: Char;
begin
  Result := 'No such user ' + FRequest;
  while (Length(FRequest) > 0) and (FRequest[Length(FRequest)] in [#$D, #$A]) do
   Delete(FRequest, Length(FRequest), 1);
  if FRequest = '' then Result := 'Users on server: '+#$D#$A+DataStore.GetUsers(false).Text
  else begin
    UInfo := DataStore.GetUserInfo(FRequest);
    if UInfo.ID <> 0 then 
    begin
      if UInfo.INVISIBLE then I := '+' else I := '-';
      if UInfo.OPER then O := '+' else O := '-';
      if UInfo.WALLOP then W := '+' else W := '-';
      if UInfo.SERVICE then S := '+' else S := '-';
      Result := Format('Nick: "%s" User ID: "%s" Real Name: "%s"', [UInfo.Nick, UInfo.UserID, UInfo.RealName]);
      Result := Result + #$D#$A + Format('Host name: %s', [UInfo.HostName]);
      Result := Result + #$D#$A + Format('Invisible: %s Operator: %s', [I, O]);
      Result := Result + #$D#$A + Format('Wallop: %s Service: %s', [W, S]);
      if UInfo.Away <> '' then
        Result := Result + #$D#$A + 'Away string: '+UInfo.Away;
    end;
  end;
  FClientSocket.SendStr(Result+#$D#$A);
  FClientSocket.Close;
  Terminate;
end;
end.